﻿using ThinkGeo.Core;

namespace Arinc.Applications.Domain.Graphics.Interfaces
{
    /// <summary>
    /// Generic background image interface
    /// </summary>
    public interface IGraphicBackgroundImage : IGraphicImage
    {
        /// <summary>
        /// Gets/Sets the background color to display around the exposed 
        /// boarder of the background image (if any)
        /// </summary>
        string BackgroundColorName { get; set; }

        /// <summary>
        /// Gets/Sets the map unit type for the image (e.g., Feet, Meters, etc)
        /// </summary>
        GeographyUnit MapUnit { get; set; }

        /// <summary>
        /// Gets the minimum X coordinate
        /// </summary>
        double MinX { get; }

        /// <summary>
        /// Gets the maximum Y coordinate
        /// </summary>
        double MaxY { get; }

        /// <summary>
        /// Gets the maximum X coordinate
        /// </summary>
        double MaxX { get; }

        /// <summary>
        /// Gets/Sets the minimum Y coordinate
        /// </summary>
        double MinY { get; }

        /// <summary>
        /// Returns the image spatial location as the specified type "T"
        /// Throws an exception if the type is not supported
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        T GetImageLocation<T>();
    }
}
