﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;
using System.Windows.Media.Imaging;

using ThinkGeo.Core;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Wpf;

namespace ThinkGeoPerformanceApp
{
    internal class CustomLayerOverlay : LayerOverlay
    {
        public GeoImage BackgroundImage { get; set; }

        public CustomLayerOverlay(GeoImage backgroundImage = null)
        {
            BackgroundImage = backgroundImage;
            if (BackgroundImage != null)
            {
                var wpfBitmap = new BitmapImage();
                wpfBitmap.BeginInit();
                wpfBitmap.StreamSource = BackgroundImage.GetImageStream();
                wpfBitmap.EndInit();

                var imageControl = new Image();
                imageControl.Source = wpfBitmap;
                imageControl.SetValue(Panel.ZIndexProperty, -100);
                OverlayCanvas.Children.Add(imageControl);
            }
        }

        public GeoImage DrawToBackground(RectangleShape worldExtent, GeographyUnit drawingMapUnit)
        {
            if (TileType != TileType.SingleTile)
            {
                return BackgroundImage;
            }

            var layerTiles = new List<LayerTile>();
            foreach (var canvasObj in OverlayCanvas.Children)
            {
                if (canvasObj is Canvas)
                {
                    foreach (var layerTileObj in (canvasObj as Canvas).Children)
                    {
                        if (layerTileObj is LayerTile)
                        {
                            layerTiles.Add(layerTileObj as LayerTile);
                        }
                    }
                }
            }

            if (layerTiles.Count > 0)
            {
                PlatformGeoCanvas geoCanvas = new PlatformGeoCanvas();
                foreach (var layerTile in layerTiles)
                {
                    if (BackgroundImage == null)
                    {
                        BackgroundImage = new GeoImage(
                            Convert.ToInt32(layerTiles[0].ActualWidth),
                            Convert.ToInt32(layerTiles[0].ActualHeight)
                        );

                        geoCanvas.BeginDrawing(BackgroundImage.NativeImage, worldExtent, drawingMapUnit);
                        geoCanvas.Clear(new GeoSolidBrush(GeoColors.Transparent));
                    }
                    else
                    {
                        geoCanvas.BeginDrawing(BackgroundImage.NativeImage, worldExtent, drawingMapUnit);
                    }

                    layerTile.Draw(geoCanvas);
                    geoCanvas.EndDrawing();
                }
            }

            return BackgroundImage;
        }
    }
}
