﻿using System;
using System.Linq;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Text;
using ThinkGeo.Core;

namespace ThinkGeoPerformanceApp
{
    /// <summary>
    /// Custom Feature Layer containing GraphicPointFeature object
    /// </summary>
    public class GraphicPointFeatureLayer : FeatureLayer
    {
        #region Members

        private GraphicPointFeatureSource _graphicPointFeatureSource;

        #endregion // Members

        #region Constructors

        public GraphicPointFeatureLayer()
        {
            Id = Guid.NewGuid();

            GraphicPointStyle graphicPointStyle = new GraphicPointStyle();
            this.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(graphicPointStyle);
            this.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            _graphicPointFeatureSource = new GraphicPointFeatureSource();
            this.FeatureSource = _graphicPointFeatureSource;
        }

        #endregion // Constructors

        #region Events

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion // Events

        #region Properties

        #region Public Properties

        public Guid Id
        {
            get;
            private set;
        }

        #endregion // Public Properties

        #region Protected Properties
        #endregion // Protected Properties

        #region Private Properties
        #endregion // Private Properties

        #endregion // Properties

        #region Methods

        #region Public Methods

        /// <summary>
        /// Returns true if _graphicPointFeatureSourceData contains the GraphicPointFeature
        /// </summary>
        /// <param name="graphicPoint">the feature to find</param>
        /// <returns></returns>
        public bool Contains(MyBaseFeature graphicPoint)
        {
            bool doesContain = false;

            if (_graphicPointFeatureSource.Contains(graphicPoint))
            {
                doesContain = true;
            }

            return doesContain;
        }

        /// <summary>
        /// Adds a graphic point feature to the feature source
        /// </summary>
        /// <param name="graphicPoint">the graphic point feature to be added</param>
        public void Add(MyBaseFeature graphicPoint)
        {
            _graphicPointFeatureSource.AddGraphicPoint(graphicPoint);
        }

        /// <summary>
        /// Clears the feature source
        /// </summary>
        public void ClearAll()
        {
            _graphicPointFeatureSource.Clear();
        }

        /// <summary>
        /// Removes the graphic point feature from the feature source
        /// </summary>
        /// <param name="graphicPoint">the feature to be removed</param>
        public void Remove(MyBaseFeature graphicPoint)
        {
            _graphicPointFeatureSource.Remove(graphicPoint);
        }

        /// <summary>
        /// Gets all of the graphic point features from the feature source
        /// </summary>
        /// <returns>a collection of features from the feature source</returns>
        public Collection<Feature> GetAllGraphicPoints()
        {
            return _graphicPointFeatureSource.GetAllGraphicPoints();
        }

        /// <summary>
        /// Returns the feature in the data source with the given id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public Feature GetFeatureById(string id)
        {
            return _graphicPointFeatureSource.GetGraphicPointFromID(id);
        }

        #endregion // Public Methods

        #region Protected Methods

        /// <summary>
        /// Create the OnPropertyChanged method to raise the event
        /// </summary>
        /// <param name="name"></param>
        protected void OnPropertyChanged(string name)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
        }

        #endregion // Protected Methods

        #region Private Methods
        #endregion // Private Methods

        #endregion // Methods
    }
}
