﻿using System.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.Core;

namespace ThinkGeoPerformanceApp
{
    /// <summary>
    /// Custom FeatureSource that contains GraphicPointFeature objects
    /// </summary>
    public class GraphicPointFeatureSource : FeatureSource
    {
        #region Members

        //private IThinkGeoLogger _logger = ThinkGeoLoggerService.Logger;
        private Collection<Feature> GraphicPointFeatureSourceData;
        private Dictionary<string, Feature> GraphicPointFeatureSourceDict;

        #endregion // Members

        #region Constructors

        /// <summary>
        /// Create a GraphicPointFeatureSource with an empty collection of features
        /// </summary>
        public GraphicPointFeatureSource()
        {
            GraphicPointFeatureSourceData = new Collection<Feature>();
            GraphicPointFeatureSourceDict = new Dictionary<string, Feature>();
        }

        #endregion // Constructors

        #region Events
        #endregion // Events

        #region Properties

        #region Public Properties

        /// <summary>
        /// Gets the number of features
        /// </summary>
        public int Count
        {
            get
            {
                return GraphicPointFeatureSourceData.Count;
            }
        }

        /// <summary>
        /// Get the IsEditable property
        /// </summary>
        public override bool IsEditable
        {
            get
            {
                return true;
            }
        }

        #endregion // Public Properties

        #region Protected Properties
        #endregion // Protected Properties

        #region Private Properties
        #endregion // Private Properties

        #endregion // Properties

        #region Methods

        #region Public Methods

        /// <summary>
        /// Adds a GraphicPointFeature
        /// </summary>
        /// <param name="graphicPoint"></param>
        public void AddGraphicPoint(MyBaseFeature graphicPoint)
        {
            //string methodName = "[GraphicPointFeatureSource.AddGraphicPoint]";

            if (!GraphicPointFeatureSourceDict.ContainsKey(graphicPoint.Id))
            {
                GraphicPointFeatureSourceData.Add(graphicPoint);
                GraphicPointFeatureSourceDict.Add(graphicPoint.Id, graphicPoint);
            }
            else
            {
                //GraphicPointFeatureSourceDict[graphicPoint.Id] = graphicPoint;
            }

            //_logger.LogFineDebug(methodName, "Adding graphicPoint '{0}' to GraphicPointFeatureSourceData.", graphicPoint.ToString());
        }

        /// <summary>
        /// Clears all features from this feature source
        /// </summary>
        public void Clear()
        {
            //string methodName = "[GraphicPointFeatureSource.Clear]";

            GraphicPointFeatureSourceData.Clear();
            GraphicPointFeatureSourceDict.Clear();

            //_logger.LogFineDebug(methodName, "Clearing GraphicPointFeatureSourceData.");
        }

        /// <summary>
        /// Gets all the features from this feature source
        /// </summary>
        /// <returns></returns>
        public Collection<Feature> GetAllGraphicPoints()
        {
            return GraphicPointFeatureSourceData;
        }


        /// <summary>
        /// Gets all the features from this feature source
        /// </summary>
        /// <returns></returns>
        public MyBaseFeature GetGraphicPointFromID(string id)
        {
            Feature feature;
            GraphicPointFeatureSourceDict.TryGetValue(id, out feature);

            if (feature == null)
            {
            }

            return (MyBaseFeature)feature;
        }
        /// <summary>
        /// Removes the specified feature from this feature source
        /// </summary>
        /// <param name="graphicPoint"></param>
        public void Remove(Feature graphicPoint)
        {
            //string methodName = "[GraphicPointFeatureSource.Remove]";

            if (GraphicPointFeatureSourceDict.Remove(graphicPoint.Id) == false)
            {
                //_logger.LogFineDebug(methodName, "graphicPoint '{0}' could not be removed.", graphicPoint.ToString());
            }

            if (this.GraphicPointFeatureSourceData.Remove(graphicPoint) == false)
            {
                //_logger.LogFineDebug(methodName, "graphicPoint '{0}' could not be removed.", graphicPoint.ToString());
            }
        }

        /// <summary>
        /// Returns true if GraphicPointFeatureSourceData contains the GraphicPointFeature
        /// </summary>
        /// <param name="graphicPoint">the feature to find</param>
        /// <returns></returns>
        public bool Contains(MyBaseFeature graphicPoint)
        {
            return GraphicPointFeatureSourceDict.ContainsKey(graphicPoint.Id);
        }

        #endregion // Public Methods

        #region Protected Methods

        /// <summary>
        ///
        /// </summary>
        /// <param name="returningColumnNames"></param>
        /// <returns></returns>
        protected override Collection<Feature> GetAllFeaturesCore(IEnumerable<string> returningColumnNames)
        {
            // Return a copy of the collection. Note that creating a new 
            // collection by calling "new Collection(GraphicPointFeatureSourceData)"
            // is on bueno. This just creates a wrapper around the collection and
            // the caller can still mutate our collection.
            Collection<Feature> features = new Collection<Feature>();
            foreach (var feature in this.GraphicPointFeatureSourceData)
            {
                if (feature != null)
                {
                    features.Add(feature);
                }
                else
                {
                }
            }

            return features;
        }

        /// <summary>
        ///
        /// </summary>
        /// <returns></returns>
        protected override bool CanGetBoundingBoxQuicklyCore()
        {
            return true;
        }

        /// <summary>
        ///
        /// </summary>
        /// <returns></returns>
        protected override bool CanGetCountQuicklyCore()
        {
            return true;
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="transactions"></param>
        /// <returns></returns>
        protected override TransactionResult CommitTransactionCore(TransactionBuffer transactions)
        {
            return new TransactionResult();
        }

        /// <summary>
        ///
        /// </summary>
        /// <returns></returns>
        protected override Collection<FeatureSourceColumn> GetColumnsCore()
        {
            return base.GetColumnsCore();
        }

        /// <summary>
        /// Get the number of features contained in this feature source
        /// </summary>
        /// <returns>the number of features</returns>
        protected override long GetCountCore()
        {
            return GraphicPointFeatureSourceData.Count;
        }

        #endregion // Protected Methods

        #region Private Methods
        #endregion // Private Methods

        #endregion // Methods
    }
}
