﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.Core;

namespace ThinkGeoPerformanceApp
{
    public class GraphicPointStyle : Style
    {
        #region Members

        private GeoFont geoFont;
        private GeoSolidBrush geoSolidBrush;
        private GeoPen geoPen;

        #endregion // Members

        #region Constructors

        /// <summary>
        /// Create a GraphicPointStyle using default values
        /// </summary>
        public GraphicPointStyle()
            : this(new GeoFont(), new GeoSolidBrush(), new GeoPen())
        {

        }

        /// <summary>
        /// Create a GraphicPointStyle using the provided values
        /// </summary>
        /// <param name="geoFont">the font used by the style</param>
        /// <param name="geoSolidBrush">the brush used by the style</param>
        /// <param name="geoPen">the pen used by the style</param>
        public GraphicPointStyle(GeoFont geoFont, GeoSolidBrush geoSolidBrush, GeoPen geoPen)
        {
            this.geoFont = geoFont;
            this.geoSolidBrush = geoSolidBrush;
            this.geoPen = geoPen;
        }

        #endregion // Constructors

        #region Events
        #endregion // Events

        #region Properties

        #region Public Properties
        #endregion // Public Properties

        #region Protected Properties
        #endregion // Protected Properties

        #region Private Properties
        #endregion // Private Properties

        #endregion // Properties

        #region Methods

        #region Public Methods
        #endregion // Public Methods

        #region Protected Methods

        /// <summary>
        /// Draw the specific styles of the features
        /// </summary>
        /// <param name="features"></param>
        /// <param name="canvas"></param>
        /// <param name="labelsInThisLayer"></param>
        /// <param name="labelsInAllLayers"></param>
        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            foreach (Feature feature in features)
            {
                MyBaseFeature myFeature = MainWindow.GlobalFeatureSource.GetGraphicPointFromID(feature.Id);  //feature as MyBaseFeature;
                if (myFeature != null)
                {
                    //Draw all the styles used by this feature
                    foreach (Style style in myFeature.Styles)
                    {
                        try
                        {
                            style.Draw(new Collection<Feature>() { feature }, canvas, labelsInThisLayer, labelsInAllLayers);
                        }
                        catch (Exception exc)
                        {
                            Console.WriteLine("Exception drawing style '{0}' for point '{1}' - {2}", style.Name, myFeature.Id, exc.ToString());
                        }
                    }
                }
                else
                {
                    Console.WriteLine("Unable to get feature with id '{0}'", feature.Id);
                }
            }
        }

        #endregion // Protected Methods

        #region Private Methods
        #endregion // Private Methods

        #endregion // Methods
    }
}
