﻿using System;
using System.Text;

using ThinkGeo.Core;

using ThinkGeoPerformanceApp;

namespace Arinc.UI.Vendor.ThinkGeo.Poco
{
    /// <summary>
    /// Manages a view as an overlay with a background image and a collection of feature layers
    /// The view's background image and feature layers are added to the map when the view
    /// becomes visible.
    /// </summary>
    public class GraphicViewOverlay : LayerOverlay
    {
        #region Members

        private GraphicRasterLayer _backgroundLayer;

        #endregion // Members

        #region Constructors

        public GraphicViewOverlay()
        {
        }

        #endregion // Constructors

        #region Events
        #endregion // Events

        #region Properties

        #region Public Properties

        /// <summary>
        /// Gets the background image layer to display for this view
        /// </summary>
        public GraphicRasterLayer BackgroundLayer
        {
            get
            {
                return _backgroundLayer;
            }
            private set
            {
                _backgroundLayer = value;
            }
        }

        /// <summary>
        /// Gets the layer extent for this custom layer
        /// </summary>
        public RectangleShape LayerExtent
        {
            get
            {
                RectangleShape extent = this.GetBoundingBox();
                return extent;
            }
        }

        /// <summary>
        /// Background color to display for the view
        /// </summary>
        public string BackgroundColorName
        {
            get
            {
                return BackgroundLayer.BackgroundColorName;
            }
        }

        #endregion // Public Properties

        #region Protected Properties
        #endregion // Protected Properties

        #region Private Properties
        #endregion // Private Properties

        #endregion // Properties

        #region Methods

        #region Public Methods

        /// <summary>
        /// Add a layer to display as a background for the view
        /// </summary>
        /// <param name="backgroundLayer"></param>
        public void AddBackgroundLayer(GraphicRasterLayer backgroundLayer)
        {
            if (BackgroundLayer == null)
            {
                BackgroundLayer = backgroundLayer;
                this.Layers.Add(Guid.NewGuid().ToString(), BackgroundLayer);
                BackgroundLayer.Open();
            }
            else
            {
                throw new ArgumentException("Cannot add background layer - Background layer is already defined");
            }
        }

        /// <summary>
        /// Add a feature layer to the collection of feature layers
        /// </summary>
        /// <param name="featureLayer">the feature layer to add</param>
        public void AddFeatureLayer(GraphicPointFeatureLayer featureLayer)
        {
            this.Layers.Add(featureLayer.Id.ToString(), featureLayer);
            featureLayer.Open();
        }

        /// <summary>
        /// Returns a string representation of the data
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            StringBuilder str = new StringBuilder();

            try
            {
                str.Append("GraphicViewOverlay - ");
                str.AppendFormat("Background: '{0}'  ", BackgroundLayer?.Name);

                StringBuilder strLayers = new StringBuilder();
                foreach (var layer in this.Layers) strLayers.AppendFormat("{0}, ", layer.Name);
                str.AppendFormat("Layers: '{0}'  ", strLayers.ToString());

            }
            catch (Exception ex)
            {
                str.AppendFormat("Exception - {0}", ex.ToString());
            }

            return str.ToString();
        }

        #endregion // Public Methods

        #region Protected Methods
        #endregion // Protected Methods

        #region Private Methods
        #endregion // Private Methods

        #endregion // Methods
    }
}
