﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using ThinkGeo.Core;

namespace ThinkGeoPerformanceApp
{
    public abstract class MyBaseFeature : Feature
    {
        public Collection<ThinkGeo.Core.Style> Styles = new Collection<ThinkGeo.Core.Style>();
        public double X { get; private set; }
        public double Y { get; private set; }
        public int MapId { get; private set; }
        public int Col { get; private set; }
        public int Row { get; private set; }
        public const string COLUMN_ID = "Id";
        public const string COLUMN_STATE = "State";

        public MyBaseFeature(BaseShape shape, int mapId, double x, double y, int col, int row, string value)
            : base(shape)
        {
            CtrInit(mapId, x, y, col, row, value);
        }

        public MyBaseFeature(int mapId, double x, double y, int col, int row, string value)
            : base(x, y)
        {
            CtrInit(mapId, x, y, col, row, value);
        }

        public abstract RectangleShape GetPointBoundingBox(RectangleShape currentExtent, float mapHeight, float mapWidth);

        public void SetFeatureValue(string value)
        {
            ColumnValues[COLUMN_STATE] = value;
        }

        public string GetFeatureValue()
        {
            return ColumnValues[COLUMN_STATE];
        }

        public override bool Equals(object obj)
        {
            if (obj != null && obj is MyBaseFeature)
            {
                MyBaseFeature other = obj as MyBaseFeature;
                return Id == other.Id;
            }

            return false;
        }

        public override int GetHashCode()
        {
            return Id.GetHashCode();
        }

        private void CtrInit(int mapId, double x, double y, int col, int row, string value)
        {
            MapId = mapId;
            X = x;
            Y = y;
            Col = col;
            Row = row;

            Id = $"POINT({mapId}){col}.{row}";

            ColumnValues.Add(COLUMN_ID, Id);
            ColumnValues.Add(COLUMN_STATE, value);
        }
    }
}
