﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using ThinkGeo.Core;

namespace ThinkGeoPerformanceApp
{
    public class MySymbolFeature : MyBaseFeature
    {
        private static readonly GeoImage _state0GeoImage;
        private static readonly GeoImage _state1GeoImage;
        private static readonly GeoImage _state2GeoImage;
        private static readonly GeoImage _state3GeoImage;
        private static readonly GeoImage _state00GeoImage;
        private static readonly GeoImage _state11GeoImage;
        private static readonly GeoImage _state22GeoImage;
        private static readonly GeoImage _state33GeoImage;
        //private static readonly GeoImage _stateNoGeoImage;

        public int FeatureWidthPixels { get; set; } = 16;
        public int FeatureHeightPixels { get; set; } = 16;

        static MySymbolFeature()
        {
            _state0GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorNormalSecure.png")));
            _state1GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorNormalNotSecure.png")));
            _state2GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorAlarmSecure.png")));
            _state3GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorAlarmNotSecure.png")));

            _state00GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorNormalSecure.png")));
            _state11GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorNormalNotSecure.png")));
            _state22GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorAlarmSecure.png")));
            _state33GeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("App_Data/doorAlarmNotSecure.png")));

            //_stateNoGeoImage = new GeoImage(new MemoryStream(File.ReadAllBytes("../../App_Data/NoImageAvailable.jpg")));
        }

        public MySymbolFeature(int mapId, double x, double y, int col, int row, string value)
            : base(mapId, x, y, col, row, value)
        {
            Styles.Add(CreateSymbolStyle());
        }

        public override RectangleShape GetPointBoundingBox(RectangleShape currentExtent, float mapHeight, float mapWidth)
        {
            if (currentExtent != null)
            {
                float halfImageWidthPixels = FeatureWidthPixels / 2;
                float halfImageHeightPixels = FeatureHeightPixels / 2;

                //Get the center point of the icon in screen and world coordinates
                PointShape featureWorldCenterPoint = new PointShape(X, Y);
                ScreenPointF featureScreenCenterPoint = MapUtil.ToScreenCoordinate(currentExtent, this, mapWidth, mapHeight);

                //Get the screen coordinates of the box
                //ScreenPointF upperLeftScreenPoint = new ScreenPointF(featureScreenCenterPoint.X - halfImageWidthPixels, featureScreenCenterPoint.Y - halfImageHeightPixels);
                //ScreenPointF upperRightScreenPoint = new ScreenPointF(featureScreenCenterPoint.X + halfImageWidthPixels, featureScreenCenterPoint.Y - halfImageHeightPixels);
                //ScreenPointF lowerRightScreenPoint = new ScreenPointF(featureScreenCenterPoint.X - halfImageWidthPixels, featureScreenCenterPoint.Y + halfImageHeightPixels);
                //ScreenPointF lowerLeftScreenPoint = new ScreenPointF(featureScreenCenterPoint.X + halfImageWidthPixels, featureScreenCenterPoint.Y + halfImageHeightPixels);
                ScreenPointF upperLeftScreenPoint = new ScreenPointF(featureScreenCenterPoint.X - halfImageWidthPixels, featureScreenCenterPoint.Y + halfImageHeightPixels);
                ScreenPointF upperRightScreenPoint = new ScreenPointF(featureScreenCenterPoint.X + halfImageWidthPixels, featureScreenCenterPoint.Y + halfImageHeightPixels);
                ScreenPointF lowerRightScreenPoint = new ScreenPointF(featureScreenCenterPoint.X + halfImageWidthPixels, featureScreenCenterPoint.Y - halfImageHeightPixels);
                ScreenPointF lowerLeftScreenPoint = new ScreenPointF(featureScreenCenterPoint.X - halfImageWidthPixels, featureScreenCenterPoint.Y - halfImageHeightPixels);

                //if (MainWindow._logDebug)
                //    Console.WriteLine("[GetPointBoundingBox] {0} pixel Location '{1}'  Width '{2}' Height '{3}'", Id, featureScreenCenterPoint.ToString(), lowerRightScreenPoint.X - lowerLeftScreenPoint.X, upperRightScreenPoint.Y - lowerRightScreenPoint.Y);

                //Convert the screen coordinates of the box to world coordinates
                PointShape upperLeftWorldPoint = MapUtil.ToWorldCoordinate(currentExtent, upperLeftScreenPoint, mapWidth, mapHeight);
                PointShape upperRightWorldPoint = MapUtil.ToWorldCoordinate(currentExtent, upperRightScreenPoint, mapWidth, mapHeight);
                PointShape lowerRightWorldPoint = MapUtil.ToWorldCoordinate(currentExtent, lowerRightScreenPoint, mapWidth, mapHeight);
                PointShape lowerLeftWorldPoint = MapUtil.ToWorldCoordinate(currentExtent, lowerLeftScreenPoint, mapWidth, mapHeight);

                //if (MainWindow._logDebug)
                //    Console.WriteLine("[GetPointBoundingBox] {0} map Location '{1}'  Width '{2}' Height '{3}'", Id, featureWorldCenterPoint.ToString(), lowerRightWorldPoint.X - lowerLeftWorldPoint.X, upperRightWorldPoint.Y - lowerRightWorldPoint.Y);

                //Create the bounding box using the calculated 4 points as vertices
                Collection<Vertex> vertices = new Collection<Vertex>();
                vertices.Add(new Vertex(upperLeftWorldPoint.X, upperLeftWorldPoint.Y));
                vertices.Add(new Vertex(upperRightWorldPoint.X, upperRightWorldPoint.Y));
                vertices.Add(new Vertex(lowerRightWorldPoint.X, lowerRightWorldPoint.Y));
                vertices.Add(new Vertex(lowerLeftWorldPoint.X, lowerLeftWorldPoint.Y));
                PolygonShape rectangleShape = new PolygonShape(new RingShape(vertices));

                //RectangleShape box = rectangleShape.GetBoundingBox();
                //if (MainWindow._logDebug)
                //    Console.WriteLine("[GetPointBoundingBox] {0} map bounding box '{1}'  Width '{2}'  Height '{3}'", Id, box.ToString(), box.LowerRightPoint.X - box.LowerLeftPoint.X, box.UpperRightPoint.Y - box.LowerRightPoint.Y);

                return rectangleShape.GetBoundingBox();
            }
            else
            {
                return GetBoundingBox();
            }
        }

        private ValueStyle CreateSymbolStyle()
        {
            ValueStyle valueStyle = new MyValueStyle();
            valueStyle.ColumnName = COLUMN_STATE;
            valueStyle.Name = "SymbolValueStyle";

            PointStyle ps = new PointStyle(_state0GeoImage);
            ValueItem vi = new ValueItem();
            vi.Value = "0";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            _state00GeoImage.Opacity = 0.25F;
            ps = new PointStyle(_state00GeoImage);
            vi = new ValueItem();
            vi.Value = "00";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            ps = new PointStyle(_state1GeoImage);
            vi = new ValueItem();
            vi.Value = "1";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            _state11GeoImage.Opacity = 25;
            ps = new PointStyle(_state11GeoImage);
            vi = new ValueItem();
            vi.Value = "11";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            ps = new PointStyle(_state2GeoImage);
            vi = new ValueItem();
            vi.Value = "2";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            _state22GeoImage.Opacity = 25;
            ps = new PointStyle(_state22GeoImage);
            vi = new ValueItem();
            vi.Value = "22";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            ps = new PointStyle(_state3GeoImage);
            vi = new ValueItem();
            vi.Value = "3";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            _state33GeoImage.Opacity = 25;
            ps = new PointStyle(_state33GeoImage);
            vi = new ValueItem();
            vi.Value = "33";
            vi.CustomStyles.Add(ps);
            valueStyle.ValueItems.Add(vi);

            return valueStyle;
        }


    }
}
