﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using ThinkGeo.Core;

namespace ThinkGeoPerformanceApp
{
    public class MyValueStyle : ValueStyle
    {
        protected override void DrawCore(
                                    System.Collections.Generic.IEnumerable<Feature> features,
                                    GeoCanvas canvas,
                                    System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInThisLayer,
                                    System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {
            foreach (Feature feature in features)
            {
                string fieldValue = feature.ColumnValues[this.ColumnName].Trim();
                ValueItem valueItem = GetValueItem(fieldValue);

                Feature[] tmpFeatures = new Feature[1] { feature };

                if (valueItem.CustomStyles.Count == 0)
                {
                    if (!string.IsNullOrEmpty(valueItem.Value))
                    {
                        valueItem.DefaultAreaStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                        valueItem.DefaultLineStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                        valueItem.DefaultPointStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                    }
                    else
                    {
                        //defaultStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                    }

                    valueItem.DefaultTextStyle.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                }
                else
                {
                    foreach (Style style in valueItem.CustomStyles)
                    {
                        style.Draw(tmpFeatures, canvas, labelsInThisLayer, labelsInAllLayers);
                    }
                }
            }
        }

        private ValueItem GetValueItem(string columnValue)
        {
            ValueItem valueItem = null;

            foreach (ValueItem item in ValueItems)
            {
                if (string.Compare(columnValue, item.Value, StringComparison.OrdinalIgnoreCase) == 0)
                {
                    valueItem = item;
                    break;
                }
            }

            if (valueItem == null)
            {
                valueItem = new ValueItem();
            }

            return valueItem;
        }
    }
}
